package com.docus.server.collect.dept.ws;

import com.docus.server.collect.ICollector;
import com.docus.server.collect.web.common.entity.TaskConfig;
import com.docus.server.collect.web.job.AbstractCollectJob;
import com.docus.server.collect.web.utils.PeriodTime;
import com.docus.server.sys.common.pojo.dto.DeptDTO;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 主动webservice请求科室数据job
 *
 * @see AbstractCollectJob
 * @see com.docus.server.collect.IJob
 */
@Component
public class DeptWsCollectJob extends AbstractCollectJob<DeptDTO> {
    @Resource(name = "wsDeptCollectorImpl")
    private ICollector collector;

    /**
     * xxl jobhandler 手工执行
     */
    @XxlJob("startAllDeptWsCollect")
    public void startAllDeptWsCollect() {
        super.startCollectAll(getJobParam());
    }

    /**
     * xxl jobhandler 配置定时启动，如果未完成任务跳过。
     */
    @XxlJob("startIncDeptWsCollect")
    public void startIncDeptWsCollect() {
        super.startCollectIncrement(getJobParam());
    }

    @Override
    public void doHandle(List<DeptDTO> results, String param) {
        deptService.batchInsertOrUpdatePowerDept(results);
    }

    @SuppressWarnings("unchecked")
    @Override
    public List<DeptDTO> execute(PeriodTime periodTime, TaskConfig taskConfig) {
        //需要根据不同医院去解析不同的内容。使用接口实现的方式。
        return collector.page(periodTime, taskConfig);
    }
}
