package com.docus.server.collect.user.view;

import com.docus.core.util.DateUtil;
import com.docus.core.util.Func;
import com.docus.core.util.ListUtils;
import com.docus.server.collect.ICollector;
import com.docus.server.collect.web.common.entity.TaskConfig;
import com.docus.server.collect.web.job.AbstractCollectJob;
import com.docus.server.collect.web.utils.PeriodTime;
import com.docus.server.his.service.IHisService;
import com.docus.server.sys.common.pojo.dto.UserDTO;
import com.docus.server.sys.common.pojo.entity.PowerDept;
import com.docus.server.sys.common.pojo.entity.PowerThirdLogin;
import com.docus.server.sys.common.pojo.entity.PowerUser;
import com.xxl.job.core.handler.annotation.XxlJob;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * 主动view请求用户数据job
 *
 * @see AbstractCollectJob
 * @see com.docus.server.collect.IJob
 * @see ICollector
 */
@Component
public class UserViewCollectJob extends AbstractCollectJob<UserDTO> {
    @Resource
    private IHisService hisService;

    /**
     * xxl jobhandler 手工执行，全量执行
     */
    @XxlJob("startAllUserViewCollect")
    public void startAllUserViewCollect() {
        super.startCollectAll(getJobParam());
    }

    /**
     * xxl jobhandler 配置定时启动，如果未完成任务跳过。增量收集
     */
    @XxlJob("startIncUserViewCollect")
    public void startIncUserViewCollect() {
        super.startCollectIncrement(getJobParam());
    }

    @Override
    public void doHandle(List<UserDTO> results, String param) {
        //docus-BasicDataReceive同步user逻辑
        Collection<Object> userNames = ListUtils.distinctSelect(results, UserDTO::getUserName);
        List<PowerUser> powerUsers = userService.findByList("userName", userNames);
        List<String> existUserNames = ListUtils.distinctSelect(powerUsers, PowerUser::getUserName);

        results.forEach(p -> {
            String userName = p.getUserName();
            String pwd = p.getPwd();
            if (existUserNames.contains(userName)) {
                updateThirdLogin(pwd, userName);
            } else {
                insertUserAndThirdLogin(p, userName);
            }
        });
    }

    @Override
    public List<UserDTO> execute(PeriodTime periodTime, TaskConfig taskConfig) {
        //每家医院按照固定的格式写sql，不同医院需要替换不同的sql即可。
        return hisService.getUserListView(periodTime.getPeriodStartDate(), periodTime.getPeriodEndDate(),
            taskConfig.getPageNum(), taskConfig.getPageSize());
    }

    private void insertUserAndThirdLogin(UserDTO p, String userName) {
        p.setPwd(userProperties.getPassword());

        if (Func.isEmpty(p.getAuthorId())) {
            p.setAuthorId("admin");
        }
        if (Func.isEmpty(p.getAuthorName())) {
            p.setAuthorName("admin");
        }

        if (Func.isNotBlank(p.getDeptId())) {
            PowerDept powerDept = deptService.findOneBy("deptCode", p.getDeptId());
            if (!Objects.isNull(powerDept)) {
                p.setDeptId(String.valueOf(powerDept.getDeptId()));
            }
        }
        userService.insertOrUpdatePowerUser(p);
        PowerUser powerUser = userService.findOneBy("userName", userName);

        PowerThirdLogin powerThirdLogin = new PowerThirdLogin();
        powerThirdLogin.setId(idService.getDateSeq());
        powerThirdLogin.setPowerUserId(String.valueOf(powerUser.getUserId()));
        powerThirdLogin.setUser(userName);
        powerThirdLogin.setPwd(p.getPwd());
        powerThirdLogin.setSource(0);
        powerThirdLogin.setCreateTime(DateUtil.now());
        thirdLoginService.save(powerThirdLogin);
    }

    private void updateThirdLogin(String pwd, String userName) {
        PowerThirdLogin user = thirdLoginService.findOneBy("user", userName);
        user.setPwd(pwd);
        thirdLoginService.updateById(user);
    }

}
