package com.docus.server.collect.web.service.impl;

import com.docus.core.util.DateUtil;
import com.docus.core.util.Func;
import com.docus.infrastructure.core.db.service.impl.BaseServiceImpl;
import com.docus.server.collect.web.common.entity.TaskOriginalMessage;
import com.docus.server.collect.web.convert.TaskOriginalMessageConverter;
import com.docus.server.collect.web.mapper.TaskOriginalMessageMapper;
import com.docus.server.collect.web.service.ITaskOriginalMessageService;
import com.docus.server.enums.CollectTypeEnum;
import com.docus.server.enums.StateEnum;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

@Service
public class TaskOriginalMessageServiceImpl extends BaseServiceImpl<TaskOriginalMessageMapper, TaskOriginalMessage> implements ITaskOriginalMessageService {
    @Resource
    private TaskOriginalMessageConverter converter;

    /**
     * 新增 ws xml 原始报文
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long insertTaskOriginalMessage(String json, String xml, CollectTypeEnum collectType, String retryKey) {
        TaskOriginalMessage taskOriginalMessage = converter.toConvertTaskOriginalMessageDO(json, xml, collectType, retryKey);
        super.save(taskOriginalMessage);
        return taskOriginalMessage.getId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTaskOriginalMessage(Long id, String json, String exMessageResult, StateEnum stateEnum) {
        if (null == id) {
            return;
        }
        TaskOriginalMessage taskOriginalMessage = super.getById(id);
        if (Func.isNotEmpty(taskOriginalMessage)) {
            taskOriginalMessage.setJsonStr(json);
            taskOriginalMessage.setUpdateTime(DateUtil.now());
            taskOriginalMessage.setState(stateEnum);
            if (Func.notNull(exMessageResult)) {
                taskOriginalMessage.setErrorMsg(exMessageResult.length() > 500 ? exMessageResult.substring(0, 500) : exMessageResult);
            }
            super.updateById(taskOriginalMessage);
        }
    }
}
